@@ -114,6 +114,7 @@ impl<T: af::HasAfEnum + Default> ArrayExt<T> {
     where
         T: af::RealNumber,
     {
+        debug_assert_eq!(self.0.dims(), dim4(self.len()));
         *self = Self(af::sort(&self.0, 0, ascending))
     }
 
@@ -192,42 +193,53 @@ impl ArrayExt<Complex<f64>> {
 }
 
 impl ArrayExt<u64> {
-    pub fn from_coords<C: IntoIterator<Item = Vec<u64>>>(shape: &[u64], coords: C) -> Self {
+    /// Construct a new `ArrayExt<u64>` with elements `start..end`.
+    pub fn range(start: u64, end: u64) -> Self {
+        let dims = dim4((end - start) as usize);
+        let tile = dim4(1);
+        let range: af::Array<u64> = af::iota(dims, tile).into();
+        if start == 0 {
+            range.into()
+        } else {
+            af::add(&range, &af::Array::new(&[start], dim4(1)), true).into()
+        }
+    }
+
+    /// Construct a new `ArrayExt<u64>` of offsets from a list of `Vec<u64>` coordinates.
+    ///
+    /// Panics: if any coordinate has length not equal to `shape.len()`.
+    pub fn from_coords<C: IntoIterator<Item = Vec<u64>>>(coords: C, shape: &[u64]) -> Self {
         let ndim = shape.len();
-        let coord_bounds = coord_bounds(shape);
-        let af_coord_bounds: af::Array<u64> =
-            af::Array::new(&coord_bounds, af::Dim4::new(&[ndim as u64, 1, 1, 1]));
+
+        let coords: Vec<u64> = coords
+            .into_iter()
+            .inspect(|coord| assert!(coord.len() == ndim))
+            .flatten()
+            .collect();
 
-        let coords: Vec<u64> = coords.into_iter().flatten().collect();
         let num_coords = coords.len() / ndim;
-        assert_eq!(coords.len(), num_coords * ndim);
 
         let coords = af::Array::new(
             &coords,
             af::Dim4::new(&[ndim as u64, num_coords as u64, 1, 1]),
         );
-        let offsets = af::mul(&coords, &af_coord_bounds, true);
-        let offsets = af::sum(&offsets, 0);
 
-        Self(offsets)
+        to_offsets(&coords, shape)
     }
 
-    pub fn to_coords(&self, shape: &[u64]) -> Vec<Vec<u64>> {
-        let ndim = shape.len();
-        assert_eq!(self.len() % ndim, 0);
-
+    /// Construct a new [`af::Array`] of coordinates from this `ArrayExt<u64>` of offsets.
+    pub fn to_coords(&self, shape: &[u64]) -> af::Array<u64> {
+        let ndim = shape.len() as u64;
         let coord_bounds = coord_bounds(shape);
+
         let af_coord_bounds: af::Array<u64> =
-            af::Array::new(&coord_bounds, af::Dim4::new(&[ndim as u64, 1, 1, 1]));
+            af::Array::new(&coord_bounds, af::Dim4::new(&[1, ndim, 1, 1]));
         let af_shape: af::Array<u64> =
-            af::Array::new(&shape.to_vec(), af::Dim4::new(&[1, ndim as u64, 1, 1]));
+            af::Array::new(&shape.to_vec(), af::Dim4::new(&[1, ndim, 1, 1]));
 
         let offsets = af::div(self.af(), &af_coord_bounds, true);
-        let offsets = af::modulo(&offsets, &af_shape, true);
-
-        let mut coords = vec![0u64; offsets.elements()];
-        af::transpose(&offsets, false).host(&mut coords);
-        coords.chunks(ndim).map(|coord| coord.to_vec()).collect()
+        let coords = af::modulo(&offsets, &af_shape, true);
+        af::transpose(&coords, false)
     }
 }
 
@@ -243,6 +255,19 @@ impl<T: af::HasAfEnum> ArrayInstance for ArrayExt<T> {
     }
 }
 
+impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> Add for ArrayExt<T>
+    where
+        <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
+        <T as af::Convertable>::OutType: af::ImplicitPromote<<T as af::Convertable>::OutType>,
+        <<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output: af::HasAfEnum, {
+
+    type Output = ArrayExt<<<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output>;
+
+    fn add(self, other: Self) -> Self::Output {
+        ArrayExt(af::add(&self.0, &other.0, BATCH))
+    }
+}
+
 impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> Add for &ArrayExt<T>
     where
         <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
@@ -268,7 +293,7 @@ impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> A
     }
 }
 
-impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> Sub for &ArrayExt<T>
+impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> Mul for ArrayExt<T>
     where
         <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
         <T as af::Convertable>::OutType: af::ImplicitPromote<<T as af::Convertable>::OutType>,
@@ -276,20 +301,8 @@ impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> S
 
     type Output = ArrayExt<<<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output>;
 
-    fn sub(self, other: Self) -> Self::Output {
-        ArrayExt(af::sub(&self.0, &other.0, BATCH))
-    }
-}
-
-impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> SubAssign for ArrayExt<T>
-    where
-        <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
-        <T as af::Convertable>::OutType: af::ImplicitPromote<<T as af::Convertable>::OutType>,
-        <<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output: af::HasAfEnum, {
-
-    fn sub_assign(&mut self, other: Self) {
-        let diff = &*self - &other;
-        *self = diff.type_cast();
+    fn mul(self, other: Self) -> Self::Output {
+        ArrayExt(af::mul(&self.0, &other.0, BATCH))
     }
 }
 
@@ -318,6 +331,19 @@ impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> M
     }
 }
 
+impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> Div for ArrayExt<T>
+    where
+        <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
+        <T as af::Convertable>::OutType: af::ImplicitPromote<<T as af::Convertable>::OutType>,
+        <<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output: af::HasAfEnum, {
+
+    type Output = ArrayExt<<<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output>;
+
+    fn div(self, other: Self) -> Self::Output {
+        ArrayExt(af::div(&self.0, &other.0, BATCH))
+    }
+}
+
 impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> Div for &ArrayExt<T>
     where
         <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
@@ -331,15 +357,67 @@ impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> D
     }
 }
 
-impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> DivAssign for ArrayExt<T>
+impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> Sub for ArrayExt<T>
     where
         <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
         <T as af::Convertable>::OutType: af::ImplicitPromote<<T as af::Convertable>::OutType>,
         <<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output: af::HasAfEnum, {
 
-    fn div_assign(&mut self, other: Self) {
-        let div = &*self / &other;
-        *self = div.type_cast();
+    type Output = ArrayExt<<<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output>;
+
+    fn sub(self, other: Self) -> Self::Output {
+        ArrayExt(af::sub(&self.0, &other.0, BATCH))
+    }
+}
+
+impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> Sub for &ArrayExt<T>
+    where
+        <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
+        <T as af::Convertable>::OutType: af::ImplicitPromote<<T as af::Convertable>::OutType>,
+        <<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output: af::HasAfEnum, {
+
+    type Output = ArrayExt<<<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output>;
+
+    fn sub(self, other: Self) -> Self::Output {
+        ArrayExt(af::sub(&self.0, &other.0, BATCH))
+    }
+}
+
+impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> SubAssign for ArrayExt<T>
+    where
+        <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
+        <T as af::Convertable>::OutType: af::ImplicitPromote<<T as af::Convertable>::OutType>,
+        <<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output: af::HasAfEnum, {
+
+    fn sub_assign(&mut self, other: Self) {
+        let diff = &*self - &other;
+        *self = diff.type_cast();
+    }
+}
+
+impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> Rem for ArrayExt<T>
+    where
+        <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
+        <T as af::Convertable>::OutType: af::ImplicitPromote<<T as af::Convertable>::OutType>,
+        <<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output: af::HasAfEnum, {
+
+    type Output = ArrayExt<<<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output>;
+
+    fn rem(self, other: Self) -> Self::Output {
+        ArrayExt(af::modulo(&self.0, &other.0, BATCH))
+    }
+}
+
+impl<T: af::HasAfEnum + af::ImplicitPromote<T> + af::Convertable<OutType = T>> Rem for &ArrayExt<T>
+    where
+        <T as af::ImplicitPromote<T>>::Output: af::HasAfEnum + Default,
+        <T as af::Convertable>::OutType: af::ImplicitPromote<<T as af::Convertable>::OutType>,
+        <<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output: af::HasAfEnum, {
+
+    type Output = ArrayExt<<<T as af::Convertable>::OutType as af::ImplicitPromote<<T as af::Convertable>::OutType>>::Output>;
+
+    fn rem(self, other: Self) -> Self::Output {
+        ArrayExt(af::modulo(&self.0, &other.0, BATCH))
     }
 }
 
@@ -958,18 +1036,55 @@ impl<'en> en::ToStream<'en> for ArrayExt<i64> {
     }
 }
 
-impl<T: af::HasAfEnum> fmt::Debug for ArrayExt<T> {
+impl<T: af::HasAfEnum + fmt::Display + Default + Clone> fmt::Debug for ArrayExt<T> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        fmt::Display::fmt(self, f)
+        write!(
+            f,
+            "ArrayExt<{}>({}): {}",
+            std::any::type_name::<T>(),
+            self.af().dims(),
+            self
+        )
     }
 }
 
-impl<T: af::HasAfEnum> fmt::Display for ArrayExt<T> {
+impl<T: af::HasAfEnum + fmt::Display + Default + Clone> fmt::Display for ArrayExt<T> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "ArrayExt<{}>", std::any::type_name::<T>())
+        let as_str: String = self
+            .to_vec()
+            .into_iter()
+            .map(|n| n.to_string())
+            .collect::<Vec<String>>()
+            .join(",");
+
+        write!(f, "[{}]", as_str)
     }
 }
 
+/// Return a list of `Vec<u64>` coordinates from the given [`af::Array<u64>`].
+pub fn to_coords(coords: &af::Array<u64>, ndim: usize) -> Vec<Vec<u64>> {
+    assert_eq!(coords.elements() % ndim, 0);
+    let mut to_vec = vec![0u64; coords.elements()];
+    coords.host(&mut to_vec);
+    to_vec.chunks(ndim).map(|coord| coord.to_vec()).collect()
+}
+
+/// Convert the given [`af::Array<u64>`] into a list of `Vec<u64>` coordinates.
+pub fn into_coords(coords: af::Array<u64>, ndim: usize) -> Vec<Vec<u64>> {
+    to_coords(&coords, ndim)
+}
+
+/// Convert the given [`af::Array<u64>`] into an [`ArrayExt<u64>`] of offsets using the given shape.
+pub fn to_offsets(coords: &af::Array<u64>, shape: &[u64]) -> ArrayExt<u64> {
+    let ndim = shape.len();
+    let coord_bounds = coord_bounds(shape);
+    let af_coord_bounds: af::Array<u64> = af::Array::new(&coord_bounds, dim4(ndim));
+
+    let offsets = af::mul(coords, &af_coord_bounds, true);
+    let offsets = af::sum(&offsets, 0).into();
+    ArrayExt(af::moddims(&offsets, dim4(offsets.elements())))
+}
+
 #[derive(Default)]
 struct ArrayExtVisitor<T> {
     phantom: PhantomData<T>,
@@ -1161,3 +1276,24 @@ fn coord_bounds(shape: &[u64]) -> Vec<u64> {
         .map(|axis| shape[axis + 1..].iter().product())
         .collect()
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_range() {
+        let range = ArrayExt::range(1, 10);
+        assert_eq!(range.to_vec(), (1..10).collect::<Vec<u64>>())
+    }
+
+    #[test]
+    fn test_to_coords() {
+        let offsets = ArrayExt::range(0, 5);
+        let coords = offsets.to_coords(&[5, 2]);
+        assert_eq!(
+            into_coords(coords, 2),
+            vec![vec![0, 0], vec![0, 1], vec![1, 0], vec![1, 1], vec![2, 0],]
+        )
+    }
+}
